%{
open Ast
%}

%token <int> INT
%token <string> ID
%token TRUE
%token FALSE
%token GEQ
%token LEQ
%token LT
%token GT
%token TIMES  
%token MINUS
%token DIVIDES
%token PLUS
%token LPAREN
%token RPAREN
%token FUN
%token ARROW
%token COLON
%token INT_TYPE
%token BOOL_TYPE
%token LET
%token EQUALS
%token IN
%token IF
%token THEN
%token ELSE
%token EOF

%nonassoc IN
%nonassoc ELSE
%nonassoc FUN
%nonassoc LET
%nonassoc LPAREN
%nonassoc TRUE
%nonassoc FALSE
%nonassoc INT
%nonassoc ID

%left LEQ
%left GEQ
%left LT
%left GT

%left PLUS
%left MINUS
%left TIMES
%left DIVIDES  

%left ARROW

%start <Ast.expr> prog

%%

prog:
	| e = expr; EOF { e }
	;
	
expr:
	| i = INT { Int i }
	| x = ID { Var x }
	| TRUE { Bool true }
	| FALSE { Bool false }
	| e1 = expr; b = binop; e2 = expr { Binop (b, e1, e2) }
	| LET; x = ID; COLON; t = typ; EQUALS; e1 = expr; IN; e2 = expr { Let (x, t, e1, e2) }
	| IF; e1 = expr; THEN; e2 = expr; ELSE; e3 = expr { If (e1, e2, e3) }
	| LPAREN; e=expr; RPAREN {e}
	| FUN; x = ID; COLON; t = typ; ARROW; e = expr { Func (x, t, e) }
	| e1 = expr; e2 = expr { Call (e1, e2) }
	;
	

binop:
	| GT  { Gt }
	| LT  { Lt }
	| LEQ { Leq }
	| GEQ { Geq }
	| PLUS    { Add }
	| MINUS   { Sub }
	| TIMES   { Mul }
	| DIVIDES { Div }
	;

typ:
	| INT_TYPE { IntType }
	| BOOL_TYPE { BoolType }
	| e = typ; ARROW; f = typ { FuncType(e,f) }
	;
